//----------------------------------------------------------------------------
// ObjectWindow - OWL NExt
// Copyright 1999. Yura Bidus.
// All Rights reserved.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
//  OVERVIEW
//  ~~~~~~~~
//  Syntax coloring text editor, grid.
//----------------------------------------------------------------------------
#if !defined(COOLPRJ_DEFS_H)
#define COOLPRJ_DEFS_H

#include <owl/pch.h>
#if !defined(OWL_CONTAIN_H)
#include <owl/contain.h>
#endif

#if !defined(_COOLCLASS)
#	if !defined(_COOLDLL)
#		define _COOLCLASS
#		define _COOLFUNC(r) r
#		if defined(BI_COMP_MSC)
#			ifdef _DEBUG
#				pragma comment(lib, "coolprjsd.lib")
#			else
#				pragma comment(lib, "coolprjs.lib")
#			endif
#		endif
#	else
#		define _COOLCLASS		_OWLIMPORTCLASS
#		define _COOLFUNC    _OWLIMPORTFUNC
#		if defined(BI_COMP_MSC)
#			ifdef _DEBUG
#				pragma comment(lib, "coolprjd.lib")
#			else
#				pragma comment(lib, "coolprj.lib")
#			endif
#		endif
#	endif
#endif

#if !defined(_COOLEDCLASS)
#define _COOLEDCLASS _COOLCLASS 
#endif
#if !defined(_COOLEDFUNC)
#define _COOLEDFUNC  _COOLFUNC
#endif
#if !defined(_COOLGRIDCLASS)
#define _COOLGRIDCLASS _COOLCLASS 
#endif
#if !defined(_DRAGDROPCLASS)
#define _DRAGDROPCLASS _COOLCLASS 
#endif
#if !defined(_PATTERNFUNC)
#define _PATTERNFUNC _COOLFUNC
#endif


#if defined(BI_STD_RTL)
#include <vector>
template <class T> class TCoolPtrArray: public vector<T> {
  public:
    TCoolPtrArray():vector<T>() {}
    ~TCoolPtrArray()    { Flush(true); }
    uint Size() const   { return size(); }
    int Add(T t)
      {
        push_back(t);
        return size()-1;
      }
    int AddAt(T t, int index)
      {
        insert(begin()+index, t);
        return size()-1;
      }
    void Flush(bool erase)
      {
        if(erase){
          for(iterator itr = begin(); itr < end(); itr++)
            delete *itr;
        }
        clear();
      }
    void Destroy(int index)
      {
        delete *(begin()+index);
        erase(begin()+index);
      }
};
#else
#include <owl/template.h>
template <class T> class TCoolPtrArray : public TIPtrArray<T> {};
#endif


//
//
//
#define MEMBER_METHOD_PROLOGUE(theClass, localClass, resultName) \
	theClass* resultName = \
		((theClass*)((BYTE*)this - offsetof(theClass, localClass))); \
	resultName; // avoid warning from compiler

//-----------------------------------------------------------------------------
//
//
//
inline int xisspecial(int c){
  return (unsigned) c > (unsigned) _T ('\x7f') || c == _T ('_');
}
inline int xisalpha (int c){
  return _istalpha (c) || xisspecial (c);
}
inline int xisalnum (int c){
  return _istalnum (c) || xisspecial (c);
}
//-----------------------------------------------------------------------------
_COOLFUNC(void) ExtractStrings(TStringArray& array, const TResId& resId, 
									             _TCHAR separator=_T('|'), TModule* module=0);
_COOLFUNC(TColor) GetDefColor16(int index);
_COOLFUNC(int) FindColorIndex16(const TColor& color);


/*============================================================================*/
#endif // COOLPRJ_DEFS_H
