//----------------------------------------------------------------------------
//  Project mdi
//  OWL NExt
//  Copyright 1999. All Rights reserved.
//
//  SUBSYSTEM:    mdi Application
//  FILE:         mdimdicl.cpp
//  AUTHOR:       Yura
//
//  OVERVIEW
//  ~~~~~~~~
//  Source file for implementation of TmdiMDIClient (TMDIClient).
//  (Generated by OWL 6.x Wizard for MS VC++. Version 1.5)
//----------------------------------------------------------------------------
#include "owlpch.h"

#include <owl/listbox.h>
#include <stdio.h>

#include "mdiapp.h"
#include "mdicl.h"
#include "mdimdicl.h"
#include "mdimdich.h"
#include "apxprint.h"
#include "apxprev.h"


//{{TmdiMDIClient Implementation}}


//
// Build a response table for all messages/commands handled
// by TmdiMDIClient derived from TMDIClient.
//
DEFINE_RESPONSE_TABLE1(TmdiMDIClient, TMDIClient)
//{{TmdiMDIClientRSP_TBL_BEGIN}}
  EV_COMMAND(CM_MDIFILENEW,       CmFileNew),
  EV_COMMAND(CM_MDIFILEOPEN,      CmFileOpen),
  EV_COMMAND(CM_FILECLOSE,        CmFileClose),
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CmPrintEnable),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CmPrintEnable),
  EV_WM_DROPFILES,
//{{TmdiMDIClientRSP_TBL_END}}
END_RESPONSE_TABLE;


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~~
// Construction/Destruction handling.
//
TmdiMDIClient::TmdiMDIClient(TModule* module)
:
  TMDIClient(module),
  ChildCount(0)
{

  // INSERT>> Your constructor code here.

}


TmdiMDIClient::~TmdiMDIClient()
{
  Destroy();

  // INSERT>> Your destructor code here.

}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~~
// MDIClient site initialization.
//
void TmdiMDIClient::SetupWindow()
{
  // Default SetUpWindow processing.
  //
  TMDIClient::SetupWindow();

  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);

  // Common file file flags and filters for Open/Save As dialogs.  Filename and directory are
  // computed in the member functions CmFileOpen, and CmFileSaveAs.
  //
  theApp->FileData.Flags |= OFN_FILEMUSTEXIST|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT;
  theApp->FileData.SetFilter(_T("All Files (*.*)|*.*|"));
  theApp->FileData.DefExt = _T("txt");
  // Accept files via drag/drop in the client window.
  //
  DragAcceptFiles(true);
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~~
// Menu File New command
//
void TmdiMDIClient::CmFileNew()
{
  TAPointer<_TCHAR> title = new _TCHAR[255];

  // Generate a title for the MDI child window.
  //
  wsprintf(title, _T("%d"), ++ChildCount);

  TmdiChild* child = new TmdiChild(*this, title, 0);

  // Assign icons for this child window.
  //
  child->SetIcon(GetApplication(), IDI_DOC);
  child->SetIconSm(GetApplication(), IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TmdiChild* curChild = (TmdiChild*)GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->ModifyStyle(0,WS_MAXIMIZE);

  child->Create();
}


void TmdiMDIClient::OpenFile(LPCTSTR fileName)
{
  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);

  if (fileName)
    _tcscpy(theApp->FileData.FileName, fileName);

  // Create a MDIChild window whose client is TmdiWindow.
  //
  TmdiChild* child = new TmdiChild(*this, _T(""), new TmdiWindow(0, 0, 0, 0, 0, 0, 0, theApp->FileData.FileName));

  // Assign icons for this child window.
  //
  child->SetIcon(GetApplication(), IDI_DOC);
  child->SetIconSm(GetApplication(), IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TmdiChild* curChild = (TmdiChild *)GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->ModifyStyle(0,WS_MAXIMIZE);

  child->Create();


  // Add the file to the MRU list.
  //
  theApp->SaveMenuChoice(theApp->FileData.FileName);
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~~
// Menu File Open command
//
void TmdiMDIClient::CmFileOpen()
{
  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);

  // Display standard Open dialog box to select a file name.
  //
  *(theApp->FileData.FileName) = 0;
  if (TFileOpenDialog(this, theApp->FileData).Execute() == IDOK)
    OpenFile();
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~~
// Menu File Close command
//
void TmdiMDIClient::CmFileClose()
{
  TmdiChild* curChild = (TmdiChild*)GetActiveMDIChild();

  if (curChild) 
    curChild->CloseWindow();
}

//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~
// Menu File Print command
//
void TmdiMDIClient::CmFilePrint()
{
  // Create Printer object if not already created.
  //
  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);
  if(theApp){
    if(!theApp->Printer)
      theApp->Printer = new TPrinter(theApp);

    TAPointer<_TCHAR> docName = new _TCHAR[_MAX_PATH];

    GetActiveMDIChild()->GetWindowText(docName, _MAX_PATH);

    // Create Printout window and set characteristics.
    //
    TApxPrintout printout(theApp->Printer, docName, GetActiveMDIChild()->GetClientWindow(), true);

    theApp->Printing++;

    // Bring up the Print dialog and print the document.
    //
    theApp->Printer->Print(GetWindowPtr(GetActiveWindow()), printout, true);

    theApp->Printing--;
  }
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~
// Menu File Print Setup command
//
void TmdiMDIClient::CmFilePrintSetup()
{
  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);
  if (theApp) {
    if (!theApp->Printer)
      theApp->Printer = new TPrinter(theApp);

    // Bring up the Print Setup dialog.
    //
    theApp->Printer->Setup(this);
  }
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~
// Menu File Print Preview command
//
void TmdiMDIClient::CmFilePrintPreview()
{
  TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);
  if (theApp) {
    if (!theApp->Printer)
      theApp->Printer = new TPrinter(GetApplication());

    theApp->Printing++;

    TApxPreviewWin* prevW =
           new TApxPreviewWin(Parent, theApp->Printer,
               GetActiveMDIChild()->GetClientWindow(), _T("Print Preview"),
                 new TLayoutWindow(0,0,0));
    prevW->Create();

    // Here we resize the preview window to take the size of the MainWindow, then
    // hide the MainWindow.
    //
    TFrameWindow * mainWindow =  GetApplication()->GetMainWindow();
    TRect r = mainWindow->GetWindowRect();
    prevW->MoveWindow(r);
    prevW->ShowWindow(SW_SHOWNORMAL);
    mainWindow->ShowWindow(SW_HIDE);

    GetApplication()->BeginModal(GetApplication()->GetMainWindow());

    // After the user closes the preview Window, we take its size and use it
    // to size the MainWindow, then show the MainWindow again.
    //
    r = prevW->GetWindowRect();
    mainWindow->MoveWindow(r);
    mainWindow->ShowWindow(SW_SHOWNORMAL);

    // We must destroy the preview window explicitly.  Otherwise, the window will
    // not be destroyed until it's parent the MainWindow is destroyed.
    //
    prevW->Destroy();
    delete prevW;

    theApp->Printing--;
  }
}


//--------------------------------------------------------
// TmdiMDIClient
// ~~~~~~~~~~
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TmdiMDIClient::CmPrintEnable(TCommandEnabler& tce)
{
  if (GetActiveMDIChild()) {
    TmdiAppl* theApp = TYPESAFE_DOWNCAST(GetApplication(), TmdiAppl);
    if (theApp) {
      // If we have a Printer already created just test if all is okay.
      // Otherwise create a Printer object and make sure the printer really
      // exists and then delete the Printer object.
      //
      if (!theApp->Printer) {
        theApp->Printer = new TPrinter(theApp);
        tce.Enable(!theApp->Printer->GetSetup().Error);
      }
      else
        tce.Enable(!theApp->Printer->GetSetup().Error);
    }
  }
  else
    tce.Enable(false);
}


void TmdiMDIClient::EvDropFiles(TDropInfo)
{
  Parent->ForwardMessage();
}
